package com.thoughtworks.rettig.dataLoader;

import junit.framework.*;
import java.lang.reflect.*;
import java.util.*;
import com.thoughtworks.rettig.util.*;

/**
 * A test case to test the ReflectiveObjectLoader. Test cases are composed of objects
 * arrays, primitives, and combinations of any of these. Different parameters are
 * used to test the load process. 
 */
public class ReflectiveObjectLoaderTest extends TestCase {

	public ReflectiveObjectLoaderTest(String name) {
		super(name);
	}
	public static void main (String[] args) {
		//junit.swingui.TestRunner.main(args);
		junit.textui.TestRunner.run (suite());
	}

	public static Test suite() {
		return new TestSuite(ReflectiveObjectLoaderTest.class);
	}


	public static Loggable getTestLogger(){
	    return new TestLogger();
	}

	public static Loggable getEmptyLogger(){
	    return new Loggable(){
		    public void logMsg(int level, String msg) {
			//System.out.println(msg);
		    }
		    };
	}

	public static LoadManager getTestManager(DataIteratorInterface iterator){
	    return new TestManager(iterator);
	}

	/**
	* The parameters that are used to run the test cases. These
	* parameters are used to construct and test each loaded object
	**/ 
	private static TestParameters[] getParams(){
		return new TestParameters[]{
			// new TestParameters(ArraySize, TestNumber, TestString, ExpectSuccess);
			new TestParameters(0, 0, "", true),
			new TestParameters(9, 56, "test", true),
			new TestParameters(0, -1, "", true),
			new TestParameters(-5, -20, "", false),	//can't have negative values for max array size
			new TestParameters(0,0,null, false)	//can't have null String values
		};
	}

	/**
	* A method to load a class against all TestParameters objects.
	**/
	private void testLoad(Class testClass)throws Throwable{

		TestsToRun emptyTests = new TestsToRun(){
			public void testObject(Object obj, TestParameters params){
			}
		};

		testLoad(testClass, emptyTests);

	}

	/**
	* Loads a class using a test iterator, and runs custom tests against the object
	**/

	private void testLoad(Class testClass, TestsToRun tests)throws Throwable{

		Object obj = null;					   
		TestParameters[] params = getParams();

		for (int i = 0; i < params.length; i++) {
			try {
				TestLogger logger = new TestLogger();
				obj = ReflectiveObjectLoader.initializeInstance(testClass, new TestManager(new TestIterator(params[i], logger)), logger);
				Assert.assertNotNull(obj);
				tests.testObject(obj, params[i]);
			} catch (java.lang.Throwable eThrow) {
				if (params[i].EXPECTSUCCESS) {
				    eThrow.printStackTrace();
					throw eThrow;
				}
			}

		}
	}

	public void testString()throws Throwable{

		TestsToRun tests = new TestsToRun(){
			public void testObject(Object obj, TestParameters params){
				String str = (String) obj;
				testStringValue(str,params);
			}
		};

		testLoad(String.class, tests);

	}


	public void testInteger()throws Throwable{

		TestsToRun tests = new TestsToRun(){
			public void testObject(Object obj, TestParameters params){
				testInteger((Integer)obj, params);      
			}
		};
		testLoad(Integer.class, tests);
	}

	private void testInteger(Integer intObject, TestParameters params){
		testIntValue(intObject.intValue(),params);
	}

	public void testIntegerArray()throws Throwable{
		TestsToRun tests = new TestsToRun(){
			public void testObject(Object obj, TestParameters params) {
				testIntegerArray((Integer[])obj, params);
			}
		};

		testLoad(Integer[].class, tests);
	}



	private void testIntegerArray(Integer[] intArray, TestParameters params){
		testArraySize(intArray, params);

		for (int i = 0; i < intArray.length; i++) {
			testIntValue(intArray[i].intValue(), params);
		}
	}
     

	private void testArraySize(Object arrayObject, TestParameters params){
		assertTrue(arrayObject.getClass().isArray());
		assertEquals(Array.getLength(arrayObject), params.MAXARRAYSIZE);
	}

	private void testStringValue(String data, TestParameters params){
		assertTrue(data.equals(params.TESTSTRING));
	}

	private void testIntValue(int num, TestParameters params){
		assertEquals(num, params.TESTINT);
	}

	public void testIntArray()throws Throwable{

		TestsToRun tests = new TestsToRun(){
			public void testObject(Object obj, TestParameters params){
				int[] intArray = (int[]) obj;
				testIntArray(intArray, params);
			}
		};

		testLoad(int[].class, tests);
	}

	private void testIntArray(int[] intArray, TestParameters params){
		testArraySize(intArray, params);
		for (int i = 0; i < intArray.length; i++) {
			testIntValue(intArray[i], params);
		}

	}

	public void testStringArray()throws Throwable{

		TestsToRun tests = new TestsToRun(){
			public void testObject(Object obj, TestParameters params){

				testStringArray((String[]) obj,params);
			}
		};

		testLoad(String[].class, tests);

	}

	private void testStringArray(String[] stringArray, TestParameters params){
		testArraySize(stringArray, params);

		for (int i = 0; i < stringArray.length; i++) {
			String data = stringArray[i];
			testStringValue(data, params);
		}
	}

	public void testFooObject() throws Throwable {

		TestsToRun tests = new TestsToRun(){
			public  void testObject(Object obj, TestParameters params){
				FooObject testData = (FooObject) obj;
				testIntValue(testData.getInt(), params);
				testStringValue(testData.getString(), params);
				testIntArray(testData.getIntArray(), params);
				testStringArray(testData.getStringArray(), params);
				testSubObject(testData.getSubFooObject(), params);
				testSubObjectArray(testData.getSubFooObjectArray(), params);

			}
		};
		testLoad(FooObject.class, tests);
	}



	public void testSubObject()throws Throwable{

		TestsToRun tests = new TestsToRun(){
			public void testObject(Object obj, TestParameters params){
				testSubObject((SubFooObject) obj, params);
			}
		};

		testLoad(SubFooObject.class, tests);
	}

	private void testSubObject(SubFooObject subObject, TestParameters params){
		testIntValue(subObject.getInt(), params);
		testStringValue(subObject.getString(), params);
	}

	public void testSubObjectArray()throws Throwable{
		TestsToRun tests = new TestsToRun(){
			public void testObject(Object obj, TestParameters params){  
				testSubObjectArray((SubFooObject[])obj, params);
			}
		};
		testLoad(SubFooObject[].class, tests);
	}

	private void testSubObjectArray(SubFooObject[] subArray, TestParameters params){

		testArraySize(subArray, params);

		for (int i = 0; i < subArray.length; i++) {
			testSubObject(subArray[i], params);
		}
	}


}

interface TestsToRun {
	public void testObject(Object obj, TestParameters params);
}



class TestParameters {
	final public int MAXARRAYSIZE;
	final public int TESTINT;
	final public String TESTSTRING;
	final public boolean EXPECTSUCCESS;

	public TestParameters(int arraySize, int testNumber, String testValue, boolean success){
		MAXARRAYSIZE = arraySize;
		TESTINT = testNumber;
		TESTSTRING = testValue;
		EXPECTSUCCESS = success;
	}
}

class TestIterator implements DataIteratorInterface {
	private int arraySizeCount= 0;
	private Loggable log;

	private TestParameters params; 

	public TestIterator(TestParameters params, Loggable log){
		this.params = params;
		this.log = log;
	}

      
	public boolean hasMoreDataForArray() {
		if (hasMore(arraySizeCount)) {
			arraySizeCount++;
			return true;
		} else
			arraySizeCount = 0;

		return false;
	}

	private boolean hasMore(int count){
		if (count < params.MAXARRAYSIZE)
			return true;
		else
			return false;

	}

	public String nextString() {
		log.logMsg(Loggable.DEBUG, "Read String: " + params.TESTSTRING);
		return params.TESTSTRING;
	}
	public int nextInt() {
		log.logMsg(Loggable.DEBUG, "Read Int: " + params.TESTINT);
		return params.TESTINT;
	}
}

class TestLogger implements Loggable {

	public void logMsg(int level, String msg) {
    		Assert.assertTrue("Log level must be greater than 0", level > 0);
		    Assert.assertTrue("Log level must be less than 6", level < 6);
		//System.out.println(msg);
		//if (level == Loggable.FATAL)
		//	Assert.fail(msg);
		//else
		 //       System.out.println(msg);

	}
}

class TestManager implements LoadManager{

    Hashtable hash = new Hashtable();
    DataIteratorInterface iterator;

    public TestManager(DataIteratorInterface iter){
	iterator = iter;

	LoadableClass[] classes = getLoadableClasses();

	for(int i = 0; i< classes.length; i++){

	    hash.put(classes[i].getName(), classes[i]);
	}

    }

    public void addSupportForLoadableClass(LoadableClass classToLoad){
	hash.put(classToLoad.getName(), classToLoad);
    }

    public LoadableClass getLoadableForClass(Class classToLoad){

	//System.out.println("Getting loadable: " + classToLoad.getName());
	Object obj = hash.get(classToLoad.getName());
	if(obj == null){
	    throw new java.lang.Error(classToLoad.getName() + " Not supported");
	}
	return (LoadableClass) obj;

    }

    public boolean hasMoreDataForArray(){
	return iterator.hasMoreDataForArray();
    }

    final private LoadableClass[] getLoadableClasses(){

	    LoadableClass intClass = new LoadableClass(int.class){
		    public Object load() {
			return new Integer(iterator.nextInt());
		    }
		    };

	    LoadableClass integerClass = new LoadableClass(Integer.class){
		    public Object load() {
			return new Integer(iterator.nextInt());
		    }
		    };

	    LoadableClass stringClass = new LoadableClass(String.class){
		    public Object load() {
			return iterator.nextString();
		    }
		    };

	    return new LoadableClass[]{
		intClass,
		integerClass,
		stringClass,
		new TestRecursable(int[].class),
		new TestRecursable(int[][].class),
		new TestRecursable(FooObject.class),
		new TestRecursable(Integer[].class),
		new TestRecursable(SubFooObject.class),
		new TestRecursable(String[].class),
		new TestRecursable(SubFooObject[].class)
	    };
	}

}

class TestRecursable extends RecursableClass{

    public TestRecursable(Class classToLoad){
	super(classToLoad);
    }

    public Constructor getRecursiveConstructor() {

	return this.getLoadClass().getConstructors()[0];
    }
}


